package ru.somecompany.config;

import com.rabbitmq.client.BuiltinExchangeType;
import com.rabbitmq.client.Channel;
import com.rabbitmq.client.Connection;
import com.rabbitmq.client.ConnectionFactory;
import lombok.RequiredArgsConstructor;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import ru.somecompany.config.property.RabbitProperties;

import java.io.IOException;
import java.util.concurrent.TimeoutException;

@Configuration
@RequiredArgsConstructor
public class ConnectionFactoryConfig {

    private final RabbitProperties rabbitProperties;

    @Bean
    public ConnectionFactory connectionFactory() {
        ConnectionFactory factory = new ConnectionFactory();
        factory.setHost(rabbitProperties.getHost());
        factory.setPort(rabbitProperties.getPort());
        return factory;
    }

    @Bean
    public Connection connection(ConnectionFactory connectionFactory) throws IOException, TimeoutException {
        return connectionFactory.newConnection();
    }

    @Bean
    public Channel channel(Connection connection) throws IOException {
        var exchange = rabbitProperties.getExchange();
        var queue = rabbitProperties.getQueue();
        var channel = connection.createChannel();

        channel.exchangeDeclare(exchange, BuiltinExchangeType.FANOUT);
        channel.queueDeclare(queue, true, false, true, null);
        channel.queueBind(queue, exchange, "");
        return channel;
    }
}
