package ru.somecompany.consumer;

import com.rabbitmq.client.AMQP;
import com.rabbitmq.client.Channel;
import com.rabbitmq.client.Connection;
import com.rabbitmq.client.DefaultConsumer;
import com.rabbitmq.client.Envelope;
import jakarta.annotation.PostConstruct;
import jakarta.annotation.PreDestroy;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import ru.somecompany.config.property.RabbitProperties;

import java.io.IOException;
import java.nio.charset.StandardCharsets;

@Slf4j
@Component
@RequiredArgsConstructor
public class Consumer {

    private final RabbitProperties rabbitProperties;

    private final Connection connection;
    private final Channel channel;

    @PostConstruct
    public void consume() {
        try {
            channel.basicQos(1);
            channel.basicConsume(rabbitProperties.getQueue(), false, new DefaultConsumer(channel) {
                @Override
                public void handleDelivery(String consumerTag,
                                           Envelope envelope,
                                           AMQP.BasicProperties properties,
                                           byte[] body) throws IOException {
                    long deliveryTag = envelope.getDeliveryTag();

                    String message = new String(body, StandardCharsets.UTF_8);
                    System.out.println(" [x] Received '" + message + "'");

                    var delay = rabbitProperties.getDelay();
                    try {
                        doWork(delay);
                    } finally {
                        System.out.println(" [x] Processed '" + message + "'");
                        channel.basicAck(deliveryTag, false);
                    }
                }
            });
        } catch (Exception exception) {
            log.error("Error while set up connection with rabbit", exception);
        }
    }

    private static void doWork(Integer delay) {
        if (delay > 0) {
            try {
                Thread.sleep(delay);
            } catch (InterruptedException _ignored) {
                Thread.currentThread().interrupt();
            }
        }
    }

    @PreDestroy
    public void cleanUp() throws Exception {
        if (channel != null) {
            channel.close();
        }
        if (connection != null) {
            connection.close();
        }
    }
}
